-------------------------------------------------------------------------------
--
-- Projekt         I2C SLAVE
--
-- Compiler        ALTERA ModelSim 5.8e
--                 QUARTUS II 5.0
--
-- Autor           FPGA-USER
--
-- Datum           26/08/2005
--
-- Sprache         VHDL 93
--
-- Info            I2C Package fuer Simulation
--                 enthaelt speziell Prozeduren zur Simulation
--                 von I2C-Transfers
--
-- Historie         
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.math_real.all;
use work.i2c_syn_pkg.all;

package i2c_sim_pkg is

   -- Konstanten ------------------------------------------------
   --
   -- hier die gewuenschte I2C Frequenz in Hz eintragen
   -- (nur fuer I2C-Simulations-Prozeduren)
   constant I2C_FREQ   : real := 400.0E+03;
   
   constant I2C_PERIOD : time := (1.0 / I2C_FREQ) * 1.0E3 * ms;
   
   constant NOT_ACK    : std_logic := '1';
   


   -- Prozeduren (nur fuer Simulation) --------------------------
	procedure start_cond
	(
		signal scl : out std_logic;
		signal sda : out std_logic
	);
                           
	procedure stop_cond
	(
		signal scl : out std_logic;
		signal sda : out std_logic
	);
   
	procedure send_bit
	(
		signal scl : out std_logic;
		signal sda : out std_logic;
             bit :  in std_logic
	);
   
   procedure send_byte
   (
		signal scl  : out std_logic;
		signal sda  : out std_logic;
             byte :  in BYTE
   );

   procedure check_ack
   (
		signal scl  : out   std_logic;
		signal sda  : inout std_logic;
             ack  : out   std_logic
   );
   
   procedure send_slave_adr
   (
      signal scl : out   std_logic;
      signal sda : inout std_logic;
             adr : in SLV_ADR_TYPE;
             trx : in TRX_TYPE;
             ack : out   std_logic
   );
   
end package;


library ieee;
use ieee.std_logic_arith.all;
package body i2c_sim_pkg is

   -----------------------------------
   -- START CONDITION
   -----------------------------------
	procedure start_cond
	(
		signal scl : out std_logic;
		signal sda : out std_logic
	)
   is
   begin
      sda <= 'H';
      scl <= 'H';
      wait for I2C_PERIOD/2;
      sda <= '0';
      wait for I2C_PERIOD/2;
   end;
   
   -----------------------------------
   -- STOP CONDITION
   -----------------------------------
	procedure stop_cond
	(
		signal scl : out std_logic;
		signal sda : out std_logic
	)
   is
   begin
      -- vor der eigentlichen Stop-Cond
      -- SCL und SDA auf 0 legen
      scl <= '0';
      sda <= '0';
      wait for I2C_PERIOD/2;
      scl <= 'H';
      wait for I2C_PERIOD/2;
      sda <= 'H';
   end;
   
   -----------------------------------
   -- BIT SENDEN
   -----------------------------------
	procedure send_bit
	(
		signal scl : out std_logic;
		signal sda : out std_logic;
             bit :  in std_logic
	)
   is
   begin
      scl <= '0';
      sda <= bit after 1 ns;
      wait for I2C_PERIOD / 2;
      scl <= 'H';
      wait for I2C_PERIOD / 2;
   end;
   
   -----------------------------------
   -- BYTE SENDEN (ohne start, stop...)
   -----------------------------------
	procedure send_byte
	(
		signal scl  : out std_logic;
		signal sda  : out std_logic;
             byte :  in BYTE
	)
   is
      variable bit_cnt : integer range 0 to 7;
   begin
      for bit_cnt in 7 downto 0 loop -- MSB First
         send_bit(scl, sda, byte(bit_cnt));
      end loop;
   end;

   -----------------------------------
   -- ACKNOWLEDGE vom SLAVE pruefen
   -- (Warnung im Simulator ausgeben)
   -----------------------------------
	procedure check_ack
	(
		signal scl  : out   std_logic;
		signal sda  : inout std_logic;
             ack  : out   std_logic
	)
   is
      variable ack_tmp : std_logic;
   begin
      scl <= '0';
      sda <= 'H'; -- damit Slave Ack senden kann
      wait for I2C_PERIOD / 2;
      scl <= 'H';
      wait for 1 ns; -- nur zur Sicherheit 
      ack_tmp := to_X01(sda);
      
      assert ack_tmp = '0'
         report "----------> kein Acknowledge vom SLAVE !"
            severity warning;
            
      wait for I2C_PERIOD / 2;
      ack := ack_tmp;
   end;
   
   -----------------------------------
   -- Slave-Adresse senden, LSB abh.
   -- vom Tranfer-Type,
   -- Ack wird ausgewertet
   -- ohne Start- und Stop-Cond !
   -----------------------------------
   procedure send_slave_adr
   (
      signal scl : out   std_logic;
      signal sda : inout std_logic;
             adr : in SLV_ADR_TYPE;
             trx : in TRX_TYPE;
             ack : out   std_logic
   )
   is
      variable slv_adr : BYTE;
   begin
      slv_adr := conv_std_logic_vector(adr *2, BYTE'LENGTH);
      if trx = READ_DATA then
         slv_adr := slv_adr or x"01"; -- set LSB fuer Read
      end if;
      send_byte(scl, sda, slv_adr);
      check_ack(scl, sda, ack);
   end;
   
   

end;

   


